#include "rfcomm.h"

void rfcomm_init(void)
{
	rftrx_init();
}

void rfcomm_set_channel(unsigned char channel)
{
	channel &= 0x7F;
	rftrx_reg_write(RFTRX_REG_RF_CH, &channel, 1);
}

void rfcomm_set_rx_pipe_address(unsigned char pipe, unsigned char* addr, unsigned char len)
{
	if (pipe == 0)
	{
		rftrx_reg_write(RFTRX_REG_RX_ADDR_P0, addr, len);
	} else if (pipe <= 5)
	{
		rftrx_reg_write(RFTRX_REG_RX_ADDR_P0+pipe, addr, len);
	}
}

void rfcomm_set_tx_pipe_address(unsigned char* addr, unsigned char len)
{
	rftrx_reg_write(RFTRX_REG_TX_ADDR, addr, len);
}

void rfcomm_set_address_length(unsigned char len)
{
	if ((len <= 5) && (len >= 3))
	{
		len = (len-2)&0x03;
		rftrx_reg_write(RFTRX_REG_SETUP_AW, &len ,1);
	}
}

void rfcomm_enable_pipe(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_EN_RXADDR, &en, 1);
		en |= (1<<pipe);
		rftrx_reg_write(RFTRX_REG_EN_RXADDR, &en, 1);
	}
}

void rfcomm_disable_pipe(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_EN_RXADDR, &en, 1);
		en &= ~(1<<pipe);
		rftrx_reg_write(RFTRX_REG_EN_RXADDR, &en, 1);
	}
}

void rfcomm_enable_pipe_autoack(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_EN_AA, &en, 1);
		en |= (1<<pipe);
		rftrx_reg_write(RFTRX_REG_EN_AA, &en, 1);
	}
}

void rfcomm_disable_pipe_autoack(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_EN_AA, &en, 1);
		en &= ~(1<<pipe);
		rftrx_reg_write(RFTRX_REG_EN_AA, &en, 1);
	}
}

void rfcomm_set_auto_retransmit_delay(unsigned char del)
{
	unsigned char retr;
	rftrx_reg_read(RFTRX_REG_SETUP_RETR, &retr, 1);
	retr = ((del&0x0F)<<4)|(retr&0x0F);
	rftrx_reg_write(RFTRX_REG_SETUP_RETR, &retr, 1);
}

void rfcomm_set_auto_retransmit_count(unsigned char num)
{
	unsigned char retr;
	rftrx_reg_read(RFTRX_REG_SETUP_RETR, &retr, 1);
	retr = (retr&0xF0)|(num&0x0F);
	rftrx_reg_write(RFTRX_REG_SETUP_RETR, &retr, 1);
}

void rfcomm_set_speed(unsigned char speed)
{
	unsigned char set;
	rftrx_reg_read(RFTRX_REG_RF_SETUP, &set, 1);
	if (speed == RFCOMM_SPEED_250KBPS)
		set = (set&0xD7)|(1<<5)|(0<<3);
	else if (speed == RFCOMM_SPEED_1MBPS)
		set = (set&0xD7)|(0<<5)|(0<<3);
	else if (speed == RFCOMM_SPEED_2MBPS)
		set = (set&0xD7)|(0<<5)|(1<<3);
	rftrx_reg_write(RFTRX_REG_RF_SETUP, &set, 1);
}

void rfcomm_set_output_power(unsigned char rfpwr)
{
	unsigned char set;
	rftrx_reg_read(RFTRX_REG_RF_SETUP, &set, 1);
	set = (set&0xF8)|((rfpwr&0x03)<<1);
	rftrx_reg_write(RFTRX_REG_RF_SETUP, &set, 1);
}

void rfcomm_enable_dynamic_pipe_payload_length(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_DYNPD, &en, 1);
		en |= (1<<pipe);
		rftrx_reg_write(RFTRX_REG_DYNPD, &en, 1);
	}	
}

void rfcomm_disable_dynamic_pipe_payload_length(unsigned char pipe)
{
	unsigned char en;  
	if (pipe <= 5)
	{
		rftrx_reg_read(RFTRX_REG_DYNPD, &en, 1);
		en &= ~(1<<pipe);
		rftrx_reg_write(RFTRX_REG_DYNPD, &en, 1);
	}	
}

void rfcomm_enable_dynamic_payload_length(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature |= (1<<2);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_disable_dynamic_payload_length(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature &= ~(1<<2);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_enable_ack_payload(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature |= (1<<1);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_disable_ack_payload(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature &= ~(1<<1);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_enable_dynamic_ack(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature |= (1<<0);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_disable_dynamic_ack(void)
{
	unsigned char feature;
	rftrx_reg_read(RFTRX_REG_FEATURE, &feature, 1);
	feature &= ~(1<<0);
	rftrx_reg_write(RFTRX_REG_FEATURE, &feature, 1);
}

void rfcomm_receive_mode(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<0);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_transmit_mode(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<0);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_power_up(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<1);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_power_down(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<1);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_enable_crc(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<3);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_disable_crc(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<3);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}											

void rfcomm_crc_scheme(unsigned char crc)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	if (crc == 1)
		control &= ~(1<<3);
	else if (crc == 2)
		control |= (1<<3);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_enable_rx_dr_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<6);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_disable_rx_dr_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<6);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_enable_tx_ds_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<5);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_disable_tx_ds_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<5);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_enable_max_rt_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control &= ~(1<<4);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

void rfcomm_disable_max_rt_irq(void)
{
	unsigned char control;
	rftrx_reg_read(RFTRX_REG_CONFIG, &control, 1);
	control |= (1<<4);
	rftrx_reg_write(RFTRX_REG_CONFIG, &control, 1);
}

unsigned char rfcomm_new_rx_data(void)
{
	return (rftrx_read_status()>>6)&0x01;
}

void rfcomm_clr_new_rx_data(void)
{
	unsigned char status = (1<<6);
	rftrx_reg_write(RFTRX_REG_STATUS, &status, 1);
}

unsigned char rfcomm_tx_data_sent(void)
{
	return (rftrx_read_status()>>5)&0x01;
}  

void rfcomm_clr_tx_data_sent(void)
{
	unsigned char status = (1<<5);
	rftrx_reg_write(RFTRX_REG_STATUS, &status, 1);
}

unsigned char rfcomm_max_retries(void)
{
	return (rftrx_read_status()>>4)&0x01;
}  

void rfcomm_clr_max_retries(void)
{
	unsigned char status = (1<<4);
	rftrx_reg_write(RFTRX_REG_STATUS, &status, 1);
}

unsigned char rfcomm_pipe_number(void)
{
	return (rftrx_read_status()>>1)&0x07;
}

unsigned char rfcomm_tx_full(void)
{
	return (rftrx_read_status()>>0)&0x01;
}