#include "rftrx.h"

void rftrx_init(void)
{
	// Initialize RF-clocks and internal SPI interface
	RFCON = (0<<0)|(1<<1)|(1<<2);
}

void rftrx_raw_write(unsigned char d)
{
	while((SPIRSTAT&(1<<0)) == 0);
	SPIRDAT = d;
}

unsigned char rftrx_raw_read(void)
{
	while((SPIRSTAT&(1<<2)) == 0);
	return SPIRDAT;
}

void rftrx_reg_write(unsigned char addr, unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_W_REGISTER|(addr&0x1F));
	rftrx_raw_read();
	for (i = 0; i < len; i++)
	{
		rftrx_raw_write(d[i]);
		rftrx_raw_read();
	}
	RFTRX_DESEL();
}

void rftrx_reg_read(unsigned char addr, unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_R_REGISTER|(addr&0x1F));
	rftrx_raw_read();
	for (i = 0; i < len; i++)
	{
		rftrx_raw_write(0x00);
		d[i] = rftrx_raw_read();  			
	}
	RFTRX_DESEL();
}

unsigned char* rftrx_rx_payload(unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_R_RX_PAYLOAD);
	rftrx_raw_read();
	for (i = 0;	i < len; i++)
	{
		rftrx_raw_write(0x00);
		d[i] = rftrx_raw_read();  			
	}
	RFTRX_DESEL();
	return d;	
}

void rftrx_tx_payload(unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_W_TX_PAYLOAD);
	rftrx_raw_read();
	for (i = 0;	i < len; i++)
	{
		rftrx_raw_write(d[i]);
		rftrx_raw_read();  			
	}
	RFTRX_DESEL();	
}

void rftrx_flush_tx(void)
{
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_FLUSH_TX);
	rftrx_raw_read();
	RFTRX_DESEL();	
}

void rftrx_flush_rx(void)
{
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_FLUSH_RX);
	rftrx_raw_read();
	RFTRX_DESEL();	
}	 

void rftrx_reuse_tx_pl(void)
{
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_REUSE_TX_PL);
	rftrx_raw_read();
	RFTRX_DESEL();	
} 

unsigned char rftrx_rx_pl_wid(void)
{
	unsigned d;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_REUSE_TX_PL);
	rftrx_raw_read();
	rftrx_raw_write(0x00);
	d = rftrx_raw_read();
	RFTRX_DESEL();	
	return d;
} 

void rftrx_ack_payload(unsigned char pipe, unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_W_ACK_PAYLOAD|(pipe&0x07));
	rftrx_raw_read();
	for (i = 0;	i < len; i++)
	{
		rftrx_raw_write(d[i]);
		rftrx_raw_read();  			
	}
	RFTRX_DESEL();	
}

void rftrx_tx_payload_noack(unsigned char* d, unsigned char len)
{
	unsigned char i;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_W_TX_PAYLOAD_NOACK);
	rftrx_raw_read();
	for (i = 0;	i < len; i++)
	{
		rftrx_raw_write(d[i]);
		rftrx_raw_read();  			
	}
	RFTRX_DESEL();	
}

unsigned char rftrx_read_status(void)
{
	unsigned char d;
	RFTRX_SEL();
	rftrx_raw_write(RFTRX_CMD_NOP);
	d = rftrx_raw_read();
	RFTRX_DESEL();
	
	return d;	
}
