#ifndef RFTRX_H_
#define RFTRX_H_

#include "reg24le1.h"

#define RFTRX_CMD_R_REGISTER 			0x00
#define RFTRX_CMD_W_REGISTER 			0x20
#define RFTRX_CMD_R_RX_PAYLOAD			0x61
#define RFTRX_CMD_W_TX_PAYLOAD			0xA0
#define RFTRX_CMD_FLUSH_TX				0xE1
#define RFTRX_CMD_FLUSH_RX				0xE2
#define RFTRX_CMD_REUSE_TX_PL			0xE3
#define RFTRX_CMD_R_RX_PL_WID			0x60
#define RFTRX_CMD_W_ACK_PAYLOAD			0xA8
#define RFTRX_CMD_W_TX_PAYLOAD_NOACK	0xB0
#define RFTRX_CMD_NOP					0xFF

#define RFTRX_REG_CONFIG				0x00
#define RFTRX_REG_EN_AA					0x01
#define RFTRX_REG_EN_RXADDR				0x02
#define RFTRX_REG_SETUP_AW				0x03
#define RFTRX_REG_SETUP_RETR			0x04
#define RFTRX_REG_RF_CH					0x05
#define RFTRX_REG_RF_SETUP				0x06
#define RFTRX_REG_STATUS				0x07
#define RFTRX_REG_OBSERVE_TX			0x08
#define RFTRX_REG_RPD					0x09
#define RFTRX_REG_RX_ADDR_P0			0x0A
#define RFTRX_REG_RX_ADDR_P1			0x0B
#define RFTRX_REG_RX_ADDR_P2			0x0C
#define RFTRX_REG_RX_ADDR_P3			0x0D
#define RFTRX_REG_RX_ADDR_P4			0x0E
#define RFTRX_REG_RX_ADDR_P5			0x0F
#define RFTRX_REG_TX_ADDR				0x10
#define RFTRX_REG_RX_PW_P0				0x11
#define RFTRX_REG_RX_PW_P1				0x12
#define RFTRX_REG_RX_PW_P2				0x13
#define RFTRX_REG_RX_PW_P3				0x14
#define RFTRX_REG_RX_PW_P4				0x15
#define RFTRX_REG_RX_PW_P5				0x16
#define RFTRX_REG_FIFO_STATUS			0x17
#define RFTRX_REG_ACK_PLD
#define RFTRX_REG_TX_PLD
#define RFTRX_REG_DYNPD					0x1C
#define RFTRX_REG_FEATURE				0x1D

#define RFTRX_SEL() RFCON&=~(1<<1)
#define RFTRX_DESEL() RFCON|=(1<<1)

void rftrx_init(void);
void rftrx_raw_write(unsigned char d);
unsigned char rftrx_raw_read(void);
void rftrx_reg_write(unsigned char addr, unsigned char* d, unsigned char len);
void rftrx_reg_read(unsigned char addr, unsigned char* d, unsigned char len);
unsigned char* rftrx_rx_payload(unsigned char* d, unsigned char len);
void rftrx_tx_payload(unsigned char* d, unsigned char len);
void rftrx_flush_tx(void);
void rftrx_flush_rx(void);
void rftrx_reuse_tx_pl(void);
unsigned char rftrx_rx_pl_wid(void);
void rftrx_ack_payload(unsigned char pipe, unsigned char* d, unsigned char len);
void rftrx_tx_payload_noack(unsigned char* d, unsigned char len);
unsigned char rftrx_read_status(void);

#endif