#include "spi.h"

unsigned char idata spi_parsing = 0;
unsigned char idata spi_cmd = 0;

extern volatile unsigned char idata RX_HAS_NEW_DATA;
extern volatile unsigned char idata RX_WRITE_POINTER;
extern volatile unsigned char idata RX_READ_POINTER;
extern volatile unsigned char xdata RX_BUFFER[256];
					
extern volatile unsigned char idata TX_PENDING;									  
extern volatile unsigned char idata TX_WRITE_POINTER;
extern volatile unsigned char idata TX_READ_POINTER;
extern volatile unsigned char xdata TX_BUFFER[256];

extern volatile unsigned char idata RF_ON;
extern volatile unsigned char idata RF_ID;

void spi_data(void) interrupt 10 using 3  // SPI irq
{
 	if (SPISSTAT&(1<<0)) // Slave transmission complete
	{
		switch (spi_parsing)
		{
			case 0:
				spi_cmd = SPISDAT;
				switch (spi_cmd)
				{
					case CMD_DUMMY:
						break;
					case CMD_WRITE:
						spi_parsing = 1;
						break;
					case CMD_READ:
						spi_parsing = 2;
						SPISDAT = 0; // LENGTH OF RX DATA ???
						break;
					case CMD_CANCEL:
						// Stop transmission
						break;
					case CMD_SETUP:
						spi_parsing = 3; // Wait for incoming setup byte
						break;
					default:
						break;
				}
				break;
			case 1:
				if (!TX_PENDING) // Check if a transmission is already in progress
				{
					TX_BUFFER[TX_WRITE_POINTER] = SPISDAT; // Fetch new data
					TX_WRITE_POINTER++; // Increment write point
					if ((TX_WRITE_POINTER == TX_BUFFER[0]) && (TX_WRITE_POINTER > 1)) // If length of packet matches write point
					{
						TX_PENDING = 1; // Flag for transmission
	
						spi_parsing = 255; // Goto idle
					}
				} else
				{
					spi_parsing = 255; // Goto idle
				}
				break;
			case 2:
				if (RX_HAS_NEW_DATA) // Check if any data is available
				{
					SPISDAT = RX_BUFFER[RX_READ_POINTER];
					RX_READ_POINTER++;
					if (RX_READ_POINTER == RX_BUFFER[0])
					{
						RX_HAS_NEW_DATA = 0;
						spi_parsing = 255; // Goto idle
					}
				} else
				{
					SPISDAT = 0; // Transmit zero length
					spi_parsing = 255; // Goto idle
				}
				break;
			case 3:
				RF_ID = SPISDAT; // Fetch SPI data
				RF_ON = (RF_ID&(1<<7))>>6; // Mask rf enable bit, RF_ON = 2 -> Enable RF
				RF_ID &= 0x07; // Mask id

				spi_parsing = 255; // Goto idle
				break;
			case 255:
				SPISDAT = 0;
				break;
			default:
				break;
		}
	}

	if (SPISSTAT&(1<<4)) // Slave select low edge
	{
		SPISDAT = ((RF_ON&0x01)<<7); // STATUS!!!
	}

	if (SPISSTAT&(1<<5)) // Slave select high edge
	{
		spi_parsing = 0;
		// Back to sleep ?
	}
}

void spi_init(void)
{
	SPISCON0 = (0<<6)|(0<<5)|(0<<4)|(1<<0); // Enable SPI Slave, MSB first, Mode 0, IRQ on nCS low&high and transmission done
	
}