/*
 * jtag.c
 *
 * Created: 28.04.2011 20:22:27
 *  Author: Tablet
 */ 

#include "jtag.h"

void jtag_start(void)
{
	gpio_enable_gpio_pin(DBG_TCK_PIN);
	gpio_enable_gpio_pin(DBG_TDI_PIN);
	gpio_enable_gpio_pin(DBG_TDO_PIN);
	gpio_enable_gpio_pin(DBG_TMS_PIN);
	gpio_enable_gpio_pin(DBG_RESET_PIN);
	
	gpio_clr_gpio_pin(DBG_TCK_PIN); // JTAG pins low
	gpio_clr_gpio_pin(DBG_TDI_PIN);
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	
	gpio_clr_gpio_pin(DBG_RESET_PIN); // Put device into reset
	
	cpu_delay_ms(1,APPLI_CPU_SPEED); // Delay 1ms
	
	jtag_resetTAP();
}

void jtag_stop(void)
{
	gpio_enable_gpio_pin(DBG_TCK_PIN);
	gpio_enable_gpio_pin(DBG_TDI_PIN);
	gpio_enable_gpio_pin(DBG_TDO_PIN);
	gpio_enable_gpio_pin(DBG_TMS_PIN);
	gpio_enable_gpio_pin(DBG_RESET_PIN);
}

void jtag_resetTAP(void)
{
	// Reset TAP by holding TMS high for five clocks
	gpio_set_gpio_pin(DBG_TMS_PIN);
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	jtag_clk();
	// And low for one clock to enter run-test/idle
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	jtag_clk();
}

uint32_t jtag_data(uint32_t data, uint8_t bits)
{
	uint8_t i;
	uint32_t out = 0;
	
	gpio_set_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Select-DR Scan
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Capture-DR
	jtag_clk(); // Shift-DR
	
	// Clock in/out data
	for (i = 0; i < bits; i++)
	{
		if (data&(1<<i))
			gpio_set_gpio_pin(DBG_TDI_PIN);
		else
			gpio_clr_gpio_pin(DBG_TDI_PIN);
		
		// If last bit, navigate to Exit1-DR	
		if (i == bits)
			gpio_set_gpio_pin(DBG_TMS_PIN);
			
		if (gpio_pin_is_high(DBG_TDO_PIN))
			out |= (1<<i);
			
		jtag_clk();
	}
	jtag_clk(); // Update-DR
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Runs-Test/Idle
	
	return out;
}

uint8_t jtag_instr(uint8_t instruction)
{
	uint8_t i;
	uint8_t out = 0;
	
	gpio_set_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Select-DR Scan
	jtag_clk(); // Select-IR Scan
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Capture-IR
	jtag_clk(); // Shift-IR
	
	// Clock in/out data
	for (i = 0; i < 4; i++)
	{
		if (instruction&(1<<i))
			gpio_set_gpio_pin(DBG_TDI_PIN);
		else
			gpio_clr_gpio_pin(DBG_TDI_PIN);
		
		// If last bit, navigate to Exit1-IR	
		if (i == 4)
			gpio_set_gpio_pin(DBG_TMS_PIN);
			
		if (gpio_pin_is_high(DBG_TDO_PIN))
			out |= (1<<i);
			
		jtag_clk();
	}
	jtag_clk(); // Update-IR
	gpio_clr_gpio_pin(DBG_TMS_PIN);
	jtag_clk(); // Runs-Test/Idle
	
	return out;
}

void jtag_clk(void)
{
	cpu_delay_us(1,APPLI_CPU_SPEED);
	gpio_set_gpio_pin(DBG_TCK_PIN);
	cpu_delay_us(1,APPLI_CPU_SPEED);
	gpio_clr_gpio_pin(DBG_TCK_PIN);
}