/**
 * \file
 *
 * \brief Empty user application template
 *
 */

/*
 * Include header files for all drivers that have been imported from
 * AVR Software Framework (ASF).
 */
#include <asf.h>
#include "usb/usb_drv.h"
#include "usb/usbb.h"
#include "pm/power_clocks_lib.h"
#include "main.h"
#include "compiler.h"
#include "config/conf_board.h"
#include "intc.h"
#include "usb/conf_usb.h"
#include "usb/usb_task.h"
#include "usb/device_cdc_task.h"
#include "memory/smc.h"
#include "utils/cycle_counter.h"
#include "twi/twim.h"
#include <avr32/io.h>
#include <stdio.h>
#include <stdlib.h>

#define SRAM_CHIP ((void*)AVR32_EBI_CS1_ADDRESS)
#define SRAM_CHIP_SIZE (1<<smc_get_cs_size(1))

#define NAND_FLASH ((void*)AVR32_EBI_CS2_ADDRESS)
#define NAND_FLASH_SIZE (1<<smc_get_cs_size(2))

pcl_freq_param_t pcl_freq_param =
{
  .cpu_f        = APPLI_CPU_SPEED,
  .pba_f        = APPLI_PBA_SPEED,
  .osc0_f       = FOSC0,
  .osc0_startup = OSC0_STARTUP
};

uint8_t tx_buffer[1024];
uint16_t tx_wrpnt = 0;
uint16_t tx_rdpnt = 0;
uint16_t tx_count = 0;

uint8_t rx_buffer[1024];
uint16_t rx_wrpnt = 0;
uint16_t rx_rdpnt = 0;
uint16_t rx_count = 0;

void usb_print(char* data);
char* itoa(int value, char* result, int base);

int main (void)
{
	//SLEEP(AVR32_PM_SMODE_STATIC);
	// Import the Exception Vector Base Address.
	extern void _evba;

	// Load the Exception Vector Base Address in the corresponding system register.
	Set_system_register(AVR32_EVBA, (int)&_evba);

	// Enable exceptions.
	Enable_global_exception();

	// Initialize interrupt handling.
	INTC_init_interrupts();
	
	board_init();
	
	
  avr32_hmatrix_scfg_t scfg;

  scfg = AVR32_HMATRIX.SCFG[AVR32_HMATRIX_SLAVE_FLASH];
  scfg.defmstr_type = AVR32_HMATRIX_DEFMSTR_TYPE_LAST_DEFAULT;
  AVR32_HMATRIX.SCFG[AVR32_HMATRIX_SLAVE_FLASH] = scfg;

  scfg = AVR32_HMATRIX.SCFG[AVR32_HMATRIX_SLAVE_EBI];
  scfg.defmstr_type = AVR32_HMATRIX_DEFMSTR_TYPE_LAST_DEFAULT;
  AVR32_HMATRIX.SCFG[AVR32_HMATRIX_SLAVE_EBI] = scfg;
	

	// Configure system clocks.
	if (pcl_configure_clocks(&pcl_freq_param) != PASS)
		return 42;

	// Shut off clocks	
	pm_disable_module(&AVR32_PM, AVR32_OCD_CLK_CPU);
	pm_disable_module(&AVR32_PM, AVR32_PDCA_CLK_HSB);
	pm_disable_module(&AVR32_PM, AVR32_DMACA_CLK_HSB);
	pm_disable_module(&AVR32_PM, AVR32_BUSMON_CLK_HSB);
	pm_disable_module(&AVR32_PM, AVR32_HRAMC0_CLK_HSB);
	pm_disable_module(&AVR32_PM, AVR32_HRAMC1_CLK_HSB);
	pm_disable_module(&AVR32_PM, AVR32_PDCA_CLK_PBA);
//	pm_disable_module(&AVR32_PM, AVR32_RTC_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_ADC_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_SPI0_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_SPI1_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_TWIM1_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_TWIS0_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_TWIS1_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_USART0_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_USART1_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_USART2_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_USART3_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_SSC_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_TC0_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_TC1_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_ABDAC_CLK_PBA);
	pm_disable_module(&AVR32_PM, AVR32_MCI_CLK_PBB);
	pm_disable_module(&AVR32_PM, AVR32_BUSMON_CLK_PBB);
	pm_disable_module(&AVR32_PM, AVR32_MSI_CLK_PBB);
	pm_disable_module(&AVR32_PM, AVR32_AES_CLK_PBB);

	// Initialize USB clock.
	pcl_configure_usb_clock();

	// Initialize USB task
	usb_task_init();
  
	// Initialize device CDC USB task
	device_cdc_task_init();
	
	
	// NOR and NAND Memory setup
	volatile uint8_t *nand_flash = NAND_FLASH;
	volatile uint16_t *sram = SRAM_CHIP;
//	smc_init(APPLI_PBA_SPEED);
	
	gpio_enable_pin_pull_up(105);
	
	static const gpio_map_t SMC_NF_EBI_GPIO_MAP =
	  {
		{ATPASTE2(EBI_NANDOE,_PIN),ATPASTE2(EBI_NANDOE,_FUNCTION)},
		{ATPASTE2(EBI_NANDWE,_PIN),ATPASTE2(EBI_NANDWE,_FUNCTION)},
	 };
	smc_init(APPLI_PBA_SPEED);

  gpio_enable_module(SMC_NF_EBI_GPIO_MAP, sizeof(SMC_NF_EBI_GPIO_MAP) / sizeof(SMC_NF_EBI_GPIO_MAP[0]));
	// Enable NAND support for NCS2&3
  AVR32_HMATRIX.sfr[AVR32_EBI_HMATRIX_NR] |= (1<<2)|(1<<3);
/*	gpio_enable_pin_pull_up(32); // NAND NWAIT
	
	gpio_enable_pin_pull_up(93); // NOR NWAIT
	gpio_clr_gpio_pin(92); // NOR Reset active
	cpu_delay_ms(1,APPLI_CPU_SPEED); // Delay 1ms
	gpio_set_gpio_pin(92); // NOR Reset inactive
	gpio_set_gpio_pin(94); // NOR Write protect off
	gpio_set_gpio_pin(95); // NOR DQ15A-1 low
*/		
	gpio_enable_gpio_pin(9);
	gpio_enable_pin_pull_up(9);
	uint8_t c;
	char itoa_buffer[100];
	while (true)
	{
		usb_task();
		device_cdc_task();
		// Test inhibit pin
		if (gpio_pin_is_high(9))
		{
			gpio_clr_gpio_pin(13);
			cpu_delay_us(1,APPLI_CPU_SPEED);
			gpio_enable_gpio_pin(13);
		}
		
		if (rx_count)
		{
			c = rx_buffer[rx_rdpnt++];
			rx_rdpnt &= 1023;
			rx_count--;
			
			if (c == 'A')
			{
				usb_print("Pow!\r\n");
			} else if (c == 'P')
			{
				gpio_clr_gpio_pin(16);
				cpu_delay_us(1,APPLI_CPU_SPEED);
				gpio_clr_gpio_pin(16);
				cpu_delay_us(1,APPLI_CPU_SPEED);
				gpio_clr_gpio_pin(8);
				cpu_delay_us(1,APPLI_CPU_SPEED);
				gpio_enable_gpio_pin(8);
				cpu_delay_us(1,APPLI_CPU_SPEED);
				gpio_enable_gpio_pin(16);
			} else if (c == 'X')
			{
				jtag_start();
				//jtag_instr(0x01); // ID
				uint32_t id = jtag_data(0,32);
				uint8_t i;
				for (i = 0; i < 32; i++)
				{
					if (id&(1<<(31-i)))
						tx_buffer[tx_wrpnt++] = '1';
					else
						tx_buffer[tx_wrpnt++] = '0';
					tx_wrpnt &= 1023;
					tx_count++;
				}
				tx_buffer[tx_wrpnt++] = '\r';
				tx_wrpnt &= 1023;
				tx_count++;
				tx_buffer[tx_wrpnt++] = '\n';
				tx_wrpnt &= 1023;
				tx_count++;
				jtag_stop();
			} else if (c == 'S')
			{
				sram[0] = 0x1337;
				sram[1] = 0x0000;
				sram[2] = 0x5555;
				sram[3] = 0xFF00;
				sram[4] = 0xAAAA;
				
				usb_print("0x");
				itoa(sram[0], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n0x");
				
				itoa(sram[1], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n0x");
				
				itoa(sram[2], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n0x");
				
				itoa(sram[3], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n0x");
				
				itoa(sram[4], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
			} else if (c == 'D')
			{
				uint32_t address;
				for (address = 0; address < 0x100000; address++)
				{
					sram[address] = (uint16_t)(~address + 0x55AA);
				}
				for (address = 0; address < 0x100000; address++)
				{
					if (sram[address] != ((uint16_t)(~address + 0x55AA)))
					{
						usb_print("Failed for address ");
						itoa(address, itoa_buffer, 10);
						usb_print(itoa_buffer);
						usb_print(". Value should have been ");
						itoa((((uint16_t)(~address)) + 0x55AA), itoa_buffer, 10);
						usb_print(itoa_buffer);
						usb_print(", but was ");
						itoa(sram[address], itoa_buffer, 10);
						usb_print(itoa_buffer);
						usb_print(".\r\n");
						break;
					}					
				}
				usb_print("SRAM Test done!\r\n");
			} else if(c == 'F')
			{
				nand_flash[0x200000] = 0x90;
				nand_flash[0x400000] = 0x00;
				itoa(nand_flash[0x00], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
				itoa(nand_flash[0x00], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
				itoa(nand_flash[0x00], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
				itoa(nand_flash[0x00], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
				itoa(nand_flash[0x00], itoa_buffer, 16);
				usb_print(itoa_buffer);
				usb_print("\r\n");
			} else if (c == 'Q')
			{
				// Init TWI
				const gpio_map_t TWIM_GPIO_MAP = {
				{AVR32_TWIMS0_TWCK_0_0_PIN, AVR32_TWIMS0_TWCK_0_0_FUNCTION},
				{AVR32_TWIMS0_TWD_0_0_PIN, AVR32_TWIMS0_TWD_0_0_FUNCTION}
				};
	
				uint8_t status;
	
				// Set TWIM options
				const twi_options_t TWIM_OPTIONS = {
					.pba_hz = APPLI_PBA_SPEED,
					.speed = TWIM_MASTER_SPEED,
					.chip = 0x47,
					.smbus = FALSE,
				};
				// TWIM gpio pins configuration
				gpio_enable_module (TWIM_GPIO_MAP,
						sizeof (TWIM_GPIO_MAP) / sizeof (TWIM_GPIO_MAP[0]));
				// Initialize as master.
				status = twim_master_init (&AVR32_TWIM0, &TWIM_OPTIONS);
				// Check whether TARGET device is connected
				if (status == STATUS_OK) {
					// display test result to user
					usb_print("TWIM: Address 0x47 OK.\r\n");
				} else {
					// display test result to user
					usb_print("TWIM: Address 0x47 FAILED.\r\n");
				}
				twi_package_t packet;
				uint8_t twi_data[10];
				
				packet.chip = 0x47;
				packet.addr = 0x01;
				packet.addr_length = 0x01;
				packet.buffer = (void*) twi_data;
				packet.length = 0x02;
				
				status = twi_master_read (&AVR32_TWIM0, &packet);
				
				if (status == STATUS_OK) {
					usb_print("OBC: \r\n");
					usb_print("High: 0x");
					itoa(twi_data[0], itoa_buffer, 16);
					usb_print(itoa_buffer);
					usb_print("\r\n");
					usb_print("Low: 0x");
					itoa(twi_data[1], itoa_buffer, 16);
					usb_print(itoa_buffer);
					usb_print("\r\n");
				} else
				{
					usb_print("Failed reading!\r\n");
				}			
				
				packet.chip = 0x49;
				packet.addr = 0x01;
				packet.addr_length = 0x01;
				packet.buffer = (void*) twi_data;
				packet.length = 0x02;
				
				status = twi_master_read (&AVR32_TWIM0, &packet);
				
				if (status == STATUS_OK) {
					usb_print("Backplane: \r\n");
					usb_print("High: 0x");
					itoa(twi_data[0], itoa_buffer, 16);
					usb_print(itoa_buffer);
					usb_print("\r\n");
					usb_print("Low: 0x");
					itoa(twi_data[1], itoa_buffer, 16);
					usb_print(itoa_buffer);
					usb_print("\r\n");
				} else
				{
					usb_print("Failed reading!\r\n");
				}			
			}
		}
	}
}

void usb_print(char* data)
{
	uint8_t i = 0;
	while (data[i] != 0)
	{	
		tx_buffer[tx_wrpnt++] = data[i++];
		tx_wrpnt &= 1023;
		tx_count++;
	}
}

/**
	 * C++ version 0.4 char* style "itoa":
	 * Written by Luks Chmela
	 * Released under GPLv3.
	 */
	char* itoa(int value, char* result, int base) {
		// check that the base if valid
		if (base < 2 || base > 36) { *result = '\0'; return result; }
	
		char* ptr = result, *ptr1 = result, tmp_char;
		int tmp_value;
	
		do {
			tmp_value = value;
			value /= base;
			*ptr++ = "zyxwvutsrqponmlkjihgfedcba9876543210123456789abcdefghijklmnopqrstuvwxyz" [35 + (tmp_value - value * base)];
		} while ( value );
	
		// Apply negative sign
		if (tmp_value < 0) *ptr++ = '-';
		*ptr-- = '\0';
		while(ptr1 < ptr) {
			tmp_char = *ptr;
			*ptr--= *ptr1;
			*ptr1++ = tmp_char;
		}
		return result;
	}
