/*This file is prepared for Doxygen automatic documentation generation.*/
/*! \file ******************************************************************
 *
 * \brief Management of the USB device CDC task.
 *
 * This file manages the USB device CDC task.
 *
 * - Compiler:           IAR EWAVR32 and GNU GCC for AVR32
 * - Supported devices:  All AVR32 devices with a USB module can be used.
 * - AppNote:
 *
 * \author               Atmel Corporation: http://www.atmel.com \n
 *                       Support and FAQ: http://support.atmel.no/
 *
 ***************************************************************************/

/* Copyright (c) 2009 Atmel Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an Atmel
 * AVR product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 *
 */

//_____  I N C L U D E S ___________________________________________________

#include <stdio.h>
#include "usb/conf_usb.h"
#include "pm/power_clocks_lib.h"


#if USB_DEVICE_FEATURE == ENABLED

#include "board.h"
#ifdef FREERTOS_USED
#include "FreeRTOS.h"
#include "task.h"
#endif
#include "usb_drv.h"
#include "gpio/gpio.h"
#include "intc.h"
#if BOARD == EVK1100 || BOARD == EVK1101
# include "joystick.h"
#endif
#include "usb_descriptors.h"
#include "usb_standard_request.h"
#include "device_cdc_task.h"
#include "uart_usb_lib.h"

//_____ M A C R O S ________________________________________________________


//_____ D E F I N I T I O N S ______________________________________________

//_____ D E C L A R A T I O N S ____________________________________________

static volatile uint16_t  sof_cnt;

//!
//! @brief This function initializes the hardware/software resources
//! required for device CDC task.
//!
void device_cdc_task_init(void)
{
	sof_cnt   =0 ;
	uart_usb_init();
	
	Usb_enable_sof_interrupt();
}

extern uint8_t tx_buffer[1024];
extern uint16_t tx_wrpnt;
extern uint16_t tx_rdpnt;
extern uint16_t tx_count;

extern uint8_t rx_buffer[1024];
extern uint16_t rx_wrpnt;
extern uint16_t rx_rdpnt;
extern uint16_t rx_count;

//!
//! @brief Entry point of the device CDC task management
//!
void device_cdc_task(void)
{
  uint8_t c;
  static Bool b_startup=true;

    // First, check the device enumeration state
    if (!Is_device_enumerated()) return;

    if( b_startup )
    {
       b_startup=false;
    }

    if( sof_cnt>=NB_MS_BEFORE_FLUSH )  //Flush buffer in Timeout
    {
       sof_cnt=0;
       uart_usb_flush();
    }

	if (uart_usb_test_hit())
	{
		if (rx_count < 1024)
		{
			rx_buffer[rx_wrpnt++] = uart_usb_getchar();
			rx_wrpnt &= 1023;
			rx_count++;
		}		
	}
	
	if (tx_count)
	{
		if( uart_usb_tx_ready() )            // "USART"-USB free ?
		{
			uart_usb_putchar(tx_buffer[tx_rdpnt++]);           // Loop back, USART to USB
			tx_rdpnt &= 1023;
			tx_count--;
		}
	}
}


//!
//! @brief usb_sof_action
//!
//! This function increments the sof_cnt counter each time
//! the USB Start-of-Frame interrupt subroutine is executed (1 ms).
//! Useful to manage time delays
//!
void usb_sof_action(void)
{
  sof_cnt++;
}

#endif  // USB_DEVICE_FEATURE == ENABLED
